import {Component, Input, OnInit, ViewChild} from '@angular/core';
import {ApexAxisChartSeries, ChartComponent, ChartType} from "ng-apexcharts";
import {ChartOptions} from "../taskgroup-activity.component";
import * as moment from "moment";
import {HistoryService, TaskgroupActivityInfo, TaskgroupPathInfo} from "../../../../api";


interface XYData {
  x: any,
  y: any
}
@Component({
  selector: 'app-heatmap-activity',
  templateUrl: './heatmap-activity.component.html',
  styleUrls: ['./heatmap-activity.component.css']
})
export class HeatmapActivityComponent implements OnInit{

  @ViewChild("chart") chart?: ChartComponent;
  public chartOptions: Partial<ChartOptions> = this.generateChartOptions()

  @Input() selectedTaskgroupPath?: TaskgroupPathInfo

  constructor(private historyService: HistoryService) {
  }

  ngOnInit() {
    this.chartOptions = this.generateChartOptions();
  }

  generateChartOptions(): Partial<ChartOptions> {
    return {
      series: this.generateSeries(),
      chart: {
        height: 350,
        type: 'heatmap',
      },
      title: {
        text: ""
      },
      dataLabels: {
        enabled: false
      },
      colors: ["#008FFB"],
    };
  }

  generateSeries() : ApexAxisChartSeries {
    const series: ApexAxisChartSeries = []

    if(this.selectedTaskgroupPath != undefined) {
      const startingDate = new Date(2023, 4, 9);
      const endDate = new Date(2023, 11, 20);
      let activityInfos: TaskgroupActivityInfo[];
      this.historyService.statisticsTaskgroupActivityTaskgroupIDStartingDateEndingDateIncludeSubTaskgroupsGet(
        this.selectedTaskgroupPath.rootTasktroup.taskgroupID,
        moment().subtract(6, "months").format("YYYY-MM-DD"),
        moment().add(2, 'days').format("YYYY-MM-DD"),
        true
      ).subscribe({
        next: resp => {
          activityInfos = resp;
          const data: XYData[][] = [];
          let currentDate: moment.Moment = moment(activityInfos[0].date).subtract(moment(activityInfos[0].date).day(), 'days');

          //Offset until data starts
          let index = currentDate.day();
          while(currentDate < moment(activityInfos[0].date)) {
            if(data[currentDate.day()] == undefined) {
              data[currentDate.day()] = [];
            }


            data[currentDate.day()][index] = {
              x: moment(currentDate).toDate(),
              y: 0
            }
            currentDate.add(1, 'days');
          }

          //inside data
          activityInfos.forEach(activityInfo => {
            const momentDate: moment.Moment = moment(activityInfo.date);
            const seriesIndex = momentDate.day();

            if(data[seriesIndex] == undefined) {
              data[seriesIndex] = [];
            }

            data[seriesIndex][index] = {
              x: activityInfo.date,
              y: activityInfo.activeMinutes
            }


            if(seriesIndex == 6) {
              index++;
            }
          })

          currentDate = moment(activityInfos[activityInfos.length-1].date);
          currentDate = currentDate.add(1, "days");
          //offset outside data
          for(let i=moment(activityInfos[activityInfos.length-1].date).day(); i<7; i++) {
            data[i][index] = {
              x: moment(currentDate).toDate(),
              y: 0
            }
            console.log(currentDate)
            currentDate = currentDate.add(1, "days");
          }


          series.push({
            name: "Saturday",
            data: data[6]
          });
          series.push({
            name: "Friday",
            data: data[5]
          });
          series.push({
            name: "Thursday",
            data: data[4]
          });
          series.push({
            name: "Wednesday",
            data: data[3]
          });
          series.push({
            name: "Tuesday",
            data: data[2]
          });
          series.push({
            name: "Monday",
            data: data[1]
          });
          series.push({
            name: "Sunday",
            data: data[0]
          });
        }
      })
      return series;
    } else {
      return series;
    }

  }

  generateData(start: moment.Moment, end: moment.Moment) {
    const data: TaskgroupActivityInfo[] = [];
    let currentDate: moment.Moment = moment(start);
    while(currentDate <= end) {
      data.push({
        date: currentDate.format("YYYY-MM-DD"),
        activeMinutes: Math.floor(Math.random() * (100 + 1))
      })
      currentDate = currentDate.add(1, 'days');
    }
    return data;
  }

  setSelectedTaskgroupPath(taskgroupPath: TaskgroupPathInfo) {
    this.selectedTaskgroupPath = taskgroupPath;
    this.chartOptions = this.generateChartOptions();
  }
}
